/*******************************************************************************
* This software program is available to you under a choice of one of two 
* licenses. You may choose to be licensed under either the GNU General Public 
* License 2.0, June 1991, available at http://www.fsf.org/copyleft/gpl.html, 
* or the Intel BSD + Patent License, the text of which follows:
* 
* Recipient has requested a license and Intel Corporation ("Intel") is willing
* to grant a license for the software entitled Linux Intel Advanced Network
* Services (iANS) (the "Licensed Program") being provided by Intel Corporation.
* The following definitions apply to this license:
* 
* "Licensed Patents" means patent claims licensable by Intel Corporation which 
* are necessarily infringed by the use of sale of the Software alone or when 
* combined with the operating system referred to below.
* 
* "Recipient" means the party to whom Intel delivers this Software.
* 
* "Licensee" means Recipient and those third parties that receive a license to 
* any operating system available under the GNU General Public License 2.0 or 
* later.
* 
* Copyright (c) 1999 - 2002 Intel Corporation.
* All rights reserved.
* 
* The license is provided to Recipient and Recipient's Licensees under the 
* following terms.
* 
* Redistribution and use in source and binary forms of the Software, with or 
* without modification, are permitted provided that the following conditions 
* are met:
* 
* Redistributions of source code of the Software may retain the above 
* copyright notice, this list of conditions and the following disclaimer.
* 
* Redistributions in binary form of the Software may reproduce the above 
* copyright notice, this list of conditions and the following disclaimer in 
* the documentation and/or materials provided with the distribution.
* 
* Neither the name of Intel Corporation nor the names of its contributors 
* shall be used to endorse or promote products derived from this Software 
* without specific prior written permission.
* 
* Intel hereby grants Recipient and Licensees a non-exclusive, worldwide, 
* royalty-free patent license under Licensed Patents to make, use, sell, offer 
* to sell, import and otherwise transfer the Software, if any, in source code 
* and object code form. This license shall include changes to the Software 
* that are error corrections or other minor changes to the Software that do 
* not add functionality or features when the Software is incorporated in any 
* version of an operating system that has been distributed under the GNU 
* General Public License 2.0 or later. This patent license shall apply to the 
* combination of the Software and any operating system licensed under the GNU 
* General Public License 2.0 or later if, at the time Intel provides the 
* Software to Recipient, such addition of the Software to the then publicly 
* available versions of such operating systems available under the GNU General 
* Public License 2.0 or later (whether in gold, beta or alpha form) causes 
* such combination to be covered by the Licensed Patents. The patent license 
* shall not apply to any other combinations which include the Software. NO 
* hardware per se is licensed hereunder.
* 
* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
* AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
* IMPLIED WARRANTIES OF MECHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
* ARE DISCLAIMED. IN NO EVENT SHALL INTEL OR IT CONTRIBUTORS BE LIABLE FOR ANY 
* DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES 
* (INCLUDING, BUT NOT LIMITED, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; 
* ANY LOSS OF USE; DATA, OR PROFITS; OR BUSINESS INTERUPTION) HOWEVER CAUSED 
* AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY OR 
* TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
* OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*******************************************************************************/

/*****************************************************************************
******************************************************************************
**                                                                          **
** INTEL CORPORATION                                                        **
**                                                                          **
** This software is supplied under the terms of the license included        **
** above.  All use of this software must be in accordance with the terms    **
** of that license.                                                         **
**                                                                          **
** THIS FILE IS USED WHEN CREATING ians_core.o. HENCE, IT SHOULD NOT BE     **
** MODIFIED!                                                                **
**                                                                          **
**  Module Name:    incg_stats.c                                            **
**                                                                          **
**  Abstract:       This module handles statistaics for iANS.               **
**                                                                          **
**  Environment:    Kernel Mode (linux 2.2.x, 2.4.x)                        **
**                                                                          **
**                  Written using Tab Size = 4, Indent Size = 4             **
**                                                                          **
**  Revision History:                                                       **
**                                                                          **
******************************************************************************
*****************************************************************************/

#include <linux/netdevice.h>

#include "ians_kcompat.h"
#include "lib/incg_defs.h"
#include "lib/incg_types.h"
#include "lib/incg_utils.h"
#include "lib/incg_open_utils.h"


/*****************************************************************************
**                                                                          **
**  Function Name:                                                          **
**                                                                          **
**  Abstract:                                                               **
**                                                                          **
**  Parameters:                                                             **
**                                                                          **
**  Assumptions:                                                            **
**                                                                          **
**  Return Value:                                                           **
**                                                                          **
**  Revision History:                                                       **
**                                                                          **
*****************************************************************************/
void iansCopyStats(pstats_t destStats, pstats_t srcStats)
{
    ASSERT(destStats && srcStats);

    destStats->rx_packets = srcStats->rx_packets;
    destStats->tx_packets = srcStats->tx_packets;
    destStats->rx_bytes = srcStats->rx_bytes;
    destStats->tx_bytes = srcStats->tx_bytes;
    destStats->rx_errors = srcStats->rx_errors;
    destStats->tx_errors = srcStats->tx_errors;
    destStats->rx_dropped = srcStats->rx_dropped;
    destStats->tx_dropped = srcStats->tx_dropped;
    destStats->multicast = srcStats->multicast;
    destStats->collisions = srcStats->collisions;
    destStats->rx_length_errors = srcStats->rx_length_errors;
    destStats->rx_over_errors = srcStats->rx_over_errors;
    destStats->rx_crc_errors = srcStats->rx_crc_errors;
    destStats->rx_frame_errors = srcStats->rx_frame_errors;
    destStats->rx_fifo_errors = srcStats->rx_fifo_errors;
    destStats->rx_missed_errors = srcStats->rx_missed_errors;

    /* detailed tx_errors */
    destStats->tx_aborted_errors = srcStats->tx_aborted_errors;
    destStats->tx_carrier_errors = srcStats->tx_carrier_errors;
    destStats->tx_fifo_errors = srcStats->tx_fifo_errors;
    destStats->tx_heartbeat_errors = srcStats->tx_heartbeat_errors;
    destStats->tx_window_errors = srcStats->tx_window_errors;

    /* for cslip etc */
    destStats->rx_compressed = srcStats->rx_compressed;
    destStats->tx_compressed = srcStats->tx_compressed;
}

/*****************************************************************************
**                                                                          **
**  Function Name:                                                          **
**                                                                          **
**  Abstract:                                                               **
**                                                                          **
**  Parameters:                                                             **
**                                                                          **
**  Assumptions:                                                            **
**                                                                          **
**  Return Value:                                                           **
**                                                                          **
**  Revision History:                                                       **
**                                                                          **
*****************************************************************************/
void iansAddStats(pstats_t destStats, pstats_t srcStats, BOOLEAN all)
{
    ASSERT(destStats && srcStats);

    if(all) {
        destStats->rx_packets += srcStats->rx_packets;
        destStats->tx_packets += srcStats->tx_packets;
    }
    destStats->rx_bytes += srcStats->rx_bytes;
    destStats->tx_bytes += srcStats->tx_bytes;
    destStats->rx_errors += srcStats->rx_errors;
    destStats->tx_errors += srcStats->tx_errors;
    destStats->rx_dropped += srcStats->rx_dropped;
    destStats->tx_dropped += srcStats->tx_dropped;
    destStats->multicast += srcStats->multicast;
    destStats->collisions += srcStats->collisions;
    destStats->rx_length_errors += srcStats->rx_length_errors;
    destStats->rx_over_errors += srcStats->rx_over_errors;
    destStats->rx_crc_errors += srcStats->rx_crc_errors;
    destStats->rx_frame_errors += srcStats->rx_frame_errors;
    destStats->rx_fifo_errors += srcStats->rx_fifo_errors;
    destStats->rx_missed_errors += srcStats->rx_missed_errors;

    /* detailed tx_errors */
    destStats->tx_aborted_errors += srcStats->tx_aborted_errors;
    destStats->tx_carrier_errors += srcStats->tx_carrier_errors;
    destStats->tx_fifo_errors += srcStats->tx_fifo_errors;
    destStats->tx_heartbeat_errors += srcStats->tx_heartbeat_errors;
    destStats->tx_window_errors += srcStats->tx_window_errors;

    /* for cslip etc */
    destStats->rx_compressed += srcStats->rx_compressed;
    destStats->tx_compressed += srcStats->tx_compressed;
}

/*****************************************************************************
**                                                                          **
**  Function Name:                                                          **
**                                                                          **
**  Abstract:                                                               **
**                                                                          **
**  Parameters:                                                             **
**                                                                          **
**  Assumptions:                                                            **
**                                                                          **
**  Return Value:                                                           **
**                                                                          **
**  Revision History:                                                       **
**                                                                          **
*****************************************************************************/
void iansSubStats(pstats_t destStats, pstats_t srcStats, BOOLEAN all)
{
    ASSERT(destStats && srcStats);

    if(all) {
        destStats->rx_packets -= srcStats->rx_packets;
        destStats->tx_packets -= srcStats->tx_packets;
    }
    destStats->rx_bytes -= srcStats->rx_bytes;
    destStats->tx_bytes -= srcStats->tx_bytes;
    destStats->rx_errors -= srcStats->rx_errors;
    destStats->tx_errors -= srcStats->tx_errors;
    destStats->rx_dropped -= srcStats->rx_dropped;
    destStats->tx_dropped -= srcStats->tx_dropped;
    destStats->multicast -= srcStats->multicast;
    destStats->collisions -= srcStats->collisions;
    destStats->rx_length_errors -= srcStats->rx_length_errors;
    destStats->rx_over_errors -= srcStats->rx_over_errors;
    destStats->rx_crc_errors -= srcStats->rx_crc_errors;
    destStats->rx_frame_errors -= srcStats->rx_frame_errors;
    destStats->rx_fifo_errors -= srcStats->rx_fifo_errors;
    destStats->rx_missed_errors -= srcStats->rx_missed_errors;

    /* detailed tx_errors */
    destStats->tx_aborted_errors -= srcStats->tx_aborted_errors;
    destStats->tx_carrier_errors -= srcStats->tx_carrier_errors;
    destStats->tx_fifo_errors -= srcStats->tx_fifo_errors;
    destStats->tx_heartbeat_errors -= srcStats->tx_heartbeat_errors;
    destStats->tx_window_errors -= srcStats->tx_window_errors;

    /* for cslip etc */
    destStats->rx_compressed -= srcStats->rx_compressed;
    destStats->tx_compressed -= srcStats->tx_compressed;
}

/*****************************************************************************
**                                                                          **
**  Function Name:                                                          **
**                                                                          **
**  Abstract:                                                               **
**                                                                          **
**  Parameters:                                                             **
**                                                                          **
**  Assumptions:                                                            **
**                                                                          **
**  Return Value:                                                           **
**                                                                          **
**  Revision History:                                                       **
**                                                                          **
*****************************************************************************/
int iansGetSizeofStats(void)
{
    return(int)sizeof(struct net_device_stats);
}

/*****************************************************************************
**                                                                          **
**  Function Name:                                                          **
**                                                                          **
**  Abstract:                                                               **
**                                                                          **
**  Parameters:                                                             **
**                                                                          **
**  Assumptions:                                                            **
**                                                                          **
**  Return Value:                                                           **
**                                                                          **
**  Revision History:                                                       **
**                                                                          **
*****************************************************************************/
void iansResetStats(pstats_t stats)
{
    ASSERT(stats);
    iAnsMemSet((void *)(stats), 0, sizeof(struct net_device_stats));
}

/*****************************************************************************
**                                                                          **
**  Function Name:                                                          **
**                                                                          **
**  Abstract:                                                               **
**                                                                          **
**  Parameters:                                                             **
**                                                                          **
**  Assumptions:                                                            **
**                                                                          **
**  Return Value:                                                           **
**                                                                          **
**  Revision History:                                                       **
**                                                                          **
*****************************************************************************/
void iansSetStats(pstats_t stats, NATIVE_ULONG rx_packets, NATIVE_ULONG tx_packets)
{
    ASSERT(stats);
    stats->rx_packets = rx_packets;
    stats->tx_packets = tx_packets;
}

/*****************************************************************************
**                                                                          **
**  Function Name:                                                          **
**                                                                          **
**  Abstract:                                                               **
**                                                                          **
**  Parameters:                                                             **
**                                                                          **
**  Assumptions:                                                            **
**                                                                          **
**  Return Value:                                                           **
**                                                                          **
**  Revision History:                                                       **
**                                                                          **
*****************************************************************************/
NATIVE_ULONG iansGetRxPacketsStats(pstats_t stats)
{
    ASSERT(stats);
    return stats->rx_packets;
}

/*****************************************************************************
**                                                                          **
**  Function Name:                                                          **
**                                                                          **
**  Abstract:                                                               **
**                                                                          **
**  Parameters:                                                             **
**                                                                          **
**  Assumptions:                                                            **
**                                                                          **
**  Return Value:                                                           **
**                                                                          **
**  Revision History:                                                       **
**                                                                          **
*****************************************************************************/
NATIVE_ULONG iansGetTxPacketsStats(pstats_t stats)
{
    ASSERT(stats);
    return stats->tx_packets;
}


/*****************************************************************************
**                                                                          **
**  Function Name:  iansGetRxBytesStats                                     **
**                                                                          **
**  Abstract:                                                               **
**                                                                          **
**  Parameters:                                                             **
**                                                                          **
**  Assumptions:                                                            **
**                                                                          **
**  Return Value:                                                           **
**                                                                          **
**  Revision History:                                                       **
**                                                                          **
*****************************************************************************/
NATIVE_ULONG iansGetRxBytesStats(pstats_t stats)
{
    ASSERT(stats);
    return stats->rx_bytes;
}

